using Revise
Revise.track(@__FILE__)

using Colors: HSV
using DualTVDD:
    ChambolleAlgorithm, ProjGradAlgorithm,
    init, step!, fetch, recover_u
using FileIO
using Plots


function InpaintProblem(img::AbstractArray{T,d}, imgmask::AbstractArray{Bool,d}, λ, β = 1e-5) where {T,d}
    Bval = inv.(imgmask .+ β .* Ref(I))
    B = Diagonal(vec(Bval))

    g = imgmask .* img
    return DualTVL1ROFOpProblem(g, B, λ)
end

function run_inpaint(img, imgmask, λ=0.5, β=0.001)
    prob = InpaintProblem(img, imgmask, λ, β)
    #alg = ChambolleAlgorithm(prob, τ=0.0001)
    alg = ProjGradAlgorithm(prob, τ=0.0001)

    ctx = init(alg)
    for i in 1:10000
        step!(ctx)
    end
    plot_inpaint(ctx)

    return ctx
end

function plot_inpaint(ctx)
    u = recover_u(fetch(ctx), ctx.algorithm.problem)

    pltg = plot(Gray.(ctx.algorithm.problem.g), c=:grayC, legend=:none, framestyle=:none, aspect_ratio=1)
    pltu = plot(Gray.(u), c=:grayC, legend=:none, framestyle=:none, aspect_ratio=1)

    #plt = plot(pltg, pltu, layout=(1,2))
    display(pltu)
end

img = Float64.(load("data/other-data-gray/RubberWhale/frame10.png"))
imgmask = rand(Bool, size(img))

#run_inpaint(f0, f1)
